import React, { useState } from 'react';
import {Context} from '../context-provider';

import '../styles/default/material-icons.css';
import '../styles/MenuBar.css';

import ArrowBack from '@material-ui/icons/ArrowBack';
import ArrowForward from '@material-ui/icons/ArrowForward';
import Refresh from '@material-ui/icons/Refresh';
import Home from '@material-ui/icons/Home';
import Folder from '@material-ui/icons/Folder';
import MoreVert from '@material-ui/icons/MoreVert';
import OpenInBrowser from '@material-ui/icons/OpenInBrowser';
import Close from '@material-ui/icons/Close';
import AccountCircle from '@material-ui/icons/AccountCircle';

import LinearProgress from '@material-ui/core/LinearProgress';


function MenuBar(){

    /* COMPONENT */

    const { appDispatch, browserState, browserDispatch, ocsApiState } = React.useContext(Context);

    const [ backButtonActive, setBackButtonActive ] = useState(false);
    const [ forwardButtonActive, setForwardButtonActive ] = useState(false);

    React.useEffect(() => {  
        
        if (browserState.canGoBack === true) setBackButtonActive(true);
        else setBackButtonActive(false);

        if (browserState.canGoForward === true) setForwardButtonActive(true);
        else setForwardButtonActive(false);
    
    },[browserState])

    function onBackButtonClick(){
        browserDispatch({type:'GO_BACK',goBack:true});
    }

    function onForwardButtonClick(){
        browserDispatch({type:'GO_FORWARD',goForward:true});
    }

    function onRefreshButtonClick(){
        browserDispatch({type:'REFRESH',refresh:true});
    }

    function onStopButtonClick(){
        browserDispatch({type:'STOP_LOADING',stopLoading:true})
    }

    function onHomeButtonClick(){
        browserDispatch({type:'GO_HOME',goHome:true});
    }

    function onCollectionsButtonClick(){
        appDispatch({type:'SHOW_DIALOG',id:'collections'})
    }

    function onLoginButtonClick(){
        browserDispatch({type:'LOAD_URL',newUrl:"https://www.pling.com/login",loadUrl:true});
    }

    /* /COMPONENT */

    /* RENDER */

    let backButtonDisplay;
    if (backButtonActive === true){
        backButtonDisplay = (
            <button title="Back" onClick={onBackButtonClick}>
                <i className="material-icons md-medium md-dark md-active"><ArrowBack/></i>
            </button>
        )
    } else {
        backButtonDisplay = (
            <button title="Back">
                <i className="material-icons md-medium md-dark md-inactive"><ArrowBack/></i>
            </button>
        )        
    }

    let forwardButtonDisplay;
    if (forwardButtonActive === true){
        forwardButtonDisplay = (
            <button title="Forward" onClick={onForwardButtonClick}>
                <i className="material-icons md-medium md-dark md-active"><ArrowForward/></i>
            </button>
        )
    } else {
        forwardButtonDisplay = (
            <button title="Forward" disabled="">
                <i className="material-icons md-medium md-dark md-inactive"><ArrowForward/></i>
            </button>
        )
    }

    let refreshButtonDisplay;
    if (browserState.loading === false){
        refreshButtonDisplay = (
            <button title="Refresh" onClick={onRefreshButtonClick}>
                <i className="material-icons md-medium md-dark md-active"><Refresh/></i>
            </button>
        )
    } else {
        refreshButtonDisplay = (
            <button title="Stop" onClick={onStopButtonClick}>
                <i className="material-icons md-medium md-dark md-active"><Close/></i>
            </button>
        )
    }

    /* /RENDER */

    let downloadItemsCounterDisplay;
    if (ocsApiState.downloadItems.length > 0){
        const filteredDownloadItems = ocsApiState.downloadItems.filter((di) => di.status === "Downloading");
        if (filteredDownloadItems.length > 0){
            downloadItemsCounterDisplay = <span className="app-badge medium-emphasis">{filteredDownloadItems.length}</span>
        }
    }

    return (
        <nav id="menu-bar">
            <ul id="main-menu">
                <li>{backButtonDisplay}</li>  
                <li>{forwardButtonDisplay}</li>
                <li>{refreshButtonDisplay}</li>
                <li>
                    <button title="Home" onClick={onHomeButtonClick}>
                        <i className="material-icons md-medium md-dark md-active">
                            <Home/>
                        </i>
                    </button>
                </li>
                <li>                
                    <button title="Collections" onClick={onCollectionsButtonClick}>
                        <i className="material-icons md-medium md-dark md-active">
                            <Folder/>
                            {downloadItemsCounterDisplay}
                        </i>
                    </button>
                </li>
                <li id="omnibox-menu"><OmniBox/></li>
                <li><UserMenu/></li>
                <li>
                    <button title="Login" onClick={onLoginButtonClick}>
                        <i className="material-icons md-medium md-dark md-active">
                            <AccountCircle/>
                        </i>
                    </button>
                </li>
            </ul>
        </nav>
    )
}

function OmniBox(){

    const { appState, browserState, browserDispatch, ocsApiState } = React.useContext(Context);
    const [ showOmniBox, setShowOmniBox ] = useState(false);

    function toggleOmniBox(){
        const newShowOmniBox = showOmniBox === false ? true : false;
        setShowOmniBox(newShowOmniBox);
    }

    function setAppHomePage(domain){
        const domainUrl = domain.url;
        const domainTitle = domain.title;
        localStorage.setItem('homePageUrl', domainUrl);
        localStorage.setItem('homePageTitle', domainTitle);
        setShowOmniBox(false);
        browserDispatch({type:'LOAD_URL',newUrl:domain.url,loadUrl:true});
    }

    function onOpenInBrowserClick(){
        console.log(browserState.url);
        ocsApiState.ocsManagerApi.send('SystemHandler::openUrl', [browserState.url])
    }

    let linearProgressionDisplay;
    if (browserState.loading === true) linearProgressionDisplay = <div id="progress-bar-container"><LinearProgress/></div>

    const domainsListDisplay = appState.domains.map((d,index) => (
        <li key={index}>
            <button onClick={() => setAppHomePage(d)} className={d.homepage === true ? "active" : ""}>
                {d.title}
            </button>
        </li>
    ));

    let omniboxCssClasses = "";
    if (ocsApiState.downloadItems){
        const filteredDownloadItems = ocsApiState.downloadItems.filter((di) => di.status === "Downloading");
        if (filteredDownloadItems.length > 0) omniboxCssClasses += " download-active";
    }

    return (
        <div id="omnibox-container">
            <div id="omnibox" className={omniboxCssClasses}>
                <div id="omnibox-wrapper">
                    <div id="omnibox-content">
                        <div></div>
                        <h3 onClick={toggleOmniBox}>{browserState.title}</h3>
                        <div id="open-in-browser-container">
                            <button title="open in browser" onClick={() => onOpenInBrowserClick()}>
                                <i className="material-icons md-small md-dark md-active"><OpenInBrowser/></i>
                            </button>
                        </div>
                    </div>
                    {linearProgressionDisplay}
                </div>
            </div>
            <div id="omnibox-palette" className={"fade-in " + (showOmniBox === true ? "active" : "inactive")}>
                <div className="palette-content">
                    <h4><i className="material-icons md-small"><Home/></i> Choose Startpage</h4>
                    <nav>
                        <ul>
                            {domainsListDisplay}
                        </ul>
                    </nav>
                </div>
            </div>
            <div id="overlay"></div>
        </div>
    )
}

function UserMenu(){

    const { appState, appDispatch, ocsApiState } = React.useContext(Context);
    const { browserDispatch } = React.useContext(Context);
    const [ showMenu, updateShowMenu ] = useState(false);

    function onToggleMenuVisiblity(){
        const newShowMenu = showMenu === true ? false : true;
        updateShowMenu(newShowMenu);
    }

    function onReportBugClick(){
        browserDispatch({type:'LOAD_URL',newUrl:appState.packageJson.bugs,loadUrl:true});
        updateShowMenu(false);
    }

    function onCheckForUpdates(){
        ocsApiState.ocsManagerApi.send('UpdateHandler::checkAppUpdate', []).then(function(res){
            console.log(res)
        });
        updateShowMenu(false);
    }

    function onAboutClick(){
        appDispatch({type:'SHOW_DIALOG',id:'about'});
        updateShowMenu(false);
    }

    let userMenuDisplay, backgroundOverlayDisplay;
    if (showMenu === true){
        userMenuDisplay = (
            <ul id="user-menu">
                <li><a onClick={() => onReportBugClick()}>Report a Bug</a></li>
                <li><a onClick={() => onCheckForUpdates()}>Check for Updates</a></li>
                <li><a onClick={() => onAboutClick()}>About This App</a></li>
            </ul>
        );
        backgroundOverlayDisplay = <div onClick={e => updateShowMenu(false)} className="background-overlay"></div>
    }

    return (
        <div id="user-menu-container">
            <button title="more operations..." onClick={onToggleMenuVisiblity}>
                <i className="material-icons md-medium md-dark md-active">
                    <MoreVert/>
                </i>
            </button>
            {userMenuDisplay}
            {backgroundOverlayDisplay}
        </div>
    )
}

export default MenuBar;
